local AddName, Ns = ...
local L = Ns.L

------------------------------
--		Options Equates		--
------------------------------
local Both, Enhanced, Basic, Damage, DamageTaken, Percentage, Rating, MainHand, OffHand, World, Realm, Modifiers, Base = 3, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 3, 2
local SameLevel, BossLevel, Regen, Casting, Critical, CritDamage, Total, Max, Average, Auto, Chance, Reduction, Complete = 1, 2, 1, 2, 1, 2, 1, 1, 2, 3, 1, 2, 1

----------------------------------
--		Text Return Formats		--
----------------------------------
local Double_Rating_Format = { "%.1f", "%.0f", "%.1f (%.0f)", }

------------------------------
--		Global Updater		--
------------------------------

Ns.armorValue, Ns.armorDebuff, Ns.Resilvalue, Ns.ohID = 0, 0, 0, 0
Ns.dodgeChance, Ns.parryChance, Ns.blockChance, Ns.avoid, Ns.defAdjust, Ns.avoidBase = 0, 0, 0, 0, 0, 0

function Ns.DefenseUpdate()

 	Ns.targetLevel = Ns.playerLevel + 3
	if UnitCanAttack("player", "target") then Ns.targetLevel = UnitLevel("target")
		if Ns.targetLevel <= 0 then Ns.targetLevel = Ns.playerLevel + 3 end
	end	

	-- Armor
	local base, effectiveArmor, armor, posBuff, negBuff = UnitArmor("player")
	Ns.armorValue = effectiveArmor
	Ns.armorDebuff = negBuff
	
	-- Resilience
	Ns.Resilvalue = GetCombatRatingBonus(15)
	
	-- Block, Parry, Dodge
	Ns.blockChance = GetBlockChance("player")
	Ns.parryChance = GetParryChance("player")
	Ns.dodgeChance = GetDodgeChance("player")
	
	-- Avoidance / Crushing
	local ohID = GetInventoryItemID("player", 17)
	Ns.avoidBlock = 0
	local classId, subId = 0, 0
	Ns.avoidBlock = 0
	
	if ohID then
		_,_,_,_,_,_,_,_,_,_,_, classId, subId = GetItemInfo(ohID)
	end
		
	if classId == 4 and subId == 6 then
		Ns.avoidBlock = Ns.blockChance
	end
				
	Ns.defAdjust = (5 + (Ns.totalDefense - (Ns.targetLevel * 5)) * 0.04)
	Ns.avoidBase = (Ns.defAdjust + Ns.dodgeChance + Ns.parryChance + Ns.blockChance)
	Ns.avoid = (Ns.defAdjust + Ns.dodgeChance + Ns.parryChance + Ns.avoidBlock)
	Ns.avoid = Ns.avoid + Ns.scorpidSting + Ns.insectSwarm + Ns.frigidDread	+ Ns.quickness
end

-- Defense
function Ns.FunctionList.DefenseStat(HUD, data, options, ...)

	local EB, skillStat, ratingStat = options.Total_Rating
	local statFormat = Double_Rating_Format[EB]
	local defenseDisplay = Ns.totalDefense
	local returnText, decimals
	local capColor = ""
	
	if options.Display_Basic then defenseDisplay = Ns.baseDefense end
	
	if defenseDisplay >= 540 then capColor = Ns.greenText end
	
	if options.Dec_Def_Def == 0 then decimals = "%.0f"
	elseif options.Dec_Def_Def == 1 then decimals = "%.1f"
	elseif options.Dec_Def_Def == 2 then decimals = "%.2f"
	else decimals = "%.3f" end	
	
	if Ns.Band(EB, Total) then returnText = capColor .. (decimals):format(defenseDisplay) end
	if Ns.Band(EB, Rating) then returnText = ("%.0f"):format(Ns.defenseRating) end
	if Ns.Band(EB, Both) then returnText = capColor .. (decimals):format(defenseDisplay) .. " |r(" .. ("%.0f"):format(Ns.defenseRating) .. ")" end		
	
	HUD:UpdateText(data, returnText)
end	
------------------------------------------------

-- Stamina
function Ns.FunctionList.Stamina(HUD, data, options, ...)

	local _, stat, posBuff, negBuff = UnitStat("Player", 3)
	local debuffColor = ""
	
	if options.Dec_Def_Stamina == 0 then decimals = "%.0f"
	elseif options.Dec_Def_Stamina == 1 then decimals = "%.1f"
	elseif options.Dec_Def_Stamina == 2 then decimals = "%.2f"
	else decimals = "%.3f" end		
	
	if negBuff < 0 then debuffColor = "|cffC41E3A" end	

	HUD:UpdateText(data, debuffColor .. (decimals):format(stat))
end

-- Armor
function Ns.FunctionList.Armor(HUD, data, options, ...)

	local EB, enhancedStat, baseStat = options.Enhanced_Base
	local returnText
	local debuffColor = ""
	
	if Ns.armorDebuff < 0 then debuffColor = Ns.redText end	
	if Ns.armorValue > 49905 then debuffColor = Ns.greenText end
	
	if Ns.Band(EB, Enhanced) then returnText = debuffColor .. ("%.0f"):format(Ns.armorValue) end
	if Ns.Band(EB, Basic) then returnText = ("%.0f"):format(Ns.armorValue) end
	if Ns.Band(EB, Both) then returnText = debuffColor .. ("%.0f"):format(Ns.armorValue) .. "/|r" .. Ns.armorValue end	
	
	HUD:UpdateText(data, returnText)
end
------------------------------------------------
	
-- Resilience
function Ns.FunctionList.Resilience(HUD, data, options, ...)

	local EB, percentStat, ratingStat = options.Crit_Damage_Taken
	local returnText, decimals, dualdec
	
	local Resilience = GetCombatRatingBonus(15)
	local resiDamage = GetCombatRatingBonus(16)
	resiDamage = resiDamage + Resilience
	
	if options.Dec_Def_Res == 0 then decimals = "%.0f%%" ; dualdec = "%.0f"
	elseif options.Dec_Def_Res == 1 then decimals = "%.1f%%" ; dualdec = "%.1f"
	elseif options.Dec_Def_Res == 2 then decimals = "%.2f%%" ; dualdec = "%.2f"
	else decimals = "%.3f%%"; dualdec = "%.3f" end		
	
	if Ns.Band(EB, Critical) then returnText = (decimals):format(Resilience) end
	if Ns.Band(EB, CritDamage) then returnText = (decimals):format(resiDamage) end
	if Ns.Band(EB, Both) then returnText = (dualdec):format(Resilience) .. "/|r" .. (decimals):format(resiDamage) end	
	
	HUD:UpdateText(data, returnText)
end
------------------------------------------------

-- Mitigation
function Ns.FunctionList.Mitigation(HUD, data, options, ...)

	local EB = options.Total_Modifiers
	local returnText, decimals
	local debuffColor = ""
	local armorReduction = (Ns.armorValue/(Ns.armorValue + 16635)) * 100
	local modifiers, bonus = 0, 0
	
	if Ns.targetLevel < 80 then armorReduction = (Ns.armorValue/(Ns.armorValue+400+85*((5.5 * Ns.targetLevel)-265.5))) * 100
	elseif Ns.targetLevel == 80 then armorReduction = Ns.armorValue/(Ns.armorValue + 15232.5) * 100 end	
	
	local mitBase = armorReduction
	
	if GetShapeshiftForm() == 2 and (Ns.classFilename == "WARRIOR") then armorReduction = armorReduction + 10
	elseif GetShapeshiftForm() == 2 and (Ns.classFilename == "DEATHKNIGHT") then armorReduction = armorReduction + Ns.impFrostPresence end
	
	modifiers = (armorReduction - (armorReduction * (Ns.blessingSanc/100)) - (armorReduction * (Ns.rightFury/100)) - (armorReduction * (Ns.divineProt/100)) - (armorReduction * (Ns.ShieldTemplar/100)) -
				(armorReduction * (Ns.divinePlea/100)) - (armorReduction * (Ns.renewedHope/100)) - (armorReduction * (Ns.bladeBarrier/100)) - (armorReduction * (Ns.runeInvicibility/100)) -
				(armorReduction * (Ns.ancestralHealing/100)) - (armorReduction * (Ns.inspiration/100)) - (armorReduction * (Ns.shamanRage/100)) - (armorReduction * (Ns.MoltenSkin/100)) -
				(armorReduction * (Ns.PrismaticCloak/100)) - (armorReduction * (Ns.frostPresence/100)) - (armorReduction * (Ns.armyDead/100)) - (armorReduction * (Ns.iceFortitude/100)) -
				(armorReduction * (Ns.iceFortitude/100)) - (armorReduction * (Ns.valorousBonus/100)) - (armorReduction * (Ns.protectorPack/100)) - (armorReduction * (Ns.vigilance/100)))
				
	armorReduction = modifiers + Ns.blessingSanc + Ns.divineProt + Ns.rightFury + Ns.ShieldTemplar + Ns.divinePlea +
					Ns.bladeBarrier + Ns.renewedHope + Ns.runeInvicibility + Ns.ancestralHealing + Ns.inspiration + 
					Ns.shamanRage + Ns.MoltenSkin + Ns.PrismaticCloak + Ns.frostPresence + Ns.armyDead + 
					Ns.iceFortitude + Ns.valorousBonus + Ns.protectorPack + Ns.vigilance
					
	bonus = Ns.blessingSanc + Ns.divineProt + Ns.rightFury + Ns.ShieldTemplar + Ns.divinePlea +
			Ns.bladeBarrier + Ns.renewedHope + Ns.runeInvicibility + Ns.ancestralHealing + Ns.inspiration + 
			Ns.shamanRage + Ns.MoltenSkin + Ns.PrismaticCloak + Ns.frostPresence + Ns.armyDead + 
			Ns.iceFortitude + Ns.valorousBonus + Ns.protectorPack + Ns.vigilance			
								
	if Ns.armorDebuff < 0 then debuffColor = Ns.redText end
	
	if armorReduction >= 75 then debuffColor = Ns.greenText; armorReduction = 75
	elseif armorReduction < 0 then armorReduction = 0; debuffColor = Ns.redText end

	if options.Dec_Def_Mit == 0 then decimals = "%.0f%%"
	elseif options.Dec_Def_Mit == 1 then decimals = "%.1f%%"
	elseif options.Dec_Def_Mit == 2 then decimals = "%.2f%%"
	else decimals = "%.3f%%" end
	
	if Ns.Band(EB, Complete) then returnText = debuffColor ..  (decimals):format(armorReduction) end
	if Ns.Band(EB, Basic) then returnText = (decimals):format(mitBase) end
	if Ns.Band(EB, Modifiers) then returnText = (decimals):format(bonus) end

	HUD:UpdateText(data, returnText)
end
------------------------------------------------

-- Hit reduction
function Ns.FunctionList.HitReduction(HUD, data, options, ...)

	local hitReduction = Ns.scorpidSting + Ns.insectSwarm + Ns.frigidDread	+ Ns.quickness + Ns.Resilvalue
	local decimals
	
	if options.Dec_Def_Hit == 0 then decimals = "%.0f%%"
	elseif options.Dec_Def_Hit == 1 then decimals = "%.1f%%"
	elseif options.Dec_Def_Hit == 2 then decimals = "%.2f%%"
	else decimals = "%.3f%%" end		
	
	HUD:UpdateText(data, (decimals):format(hitReduction))
end
------------------------------------------------

-- Crit Immunity
function Ns.FunctionList.CritReceived(HUD, data, options, ...)

	local EB, percentStat, ratingStat = options.Chance_Reduction
	local chanceGetCrit = 19
	local getCritLevel = (Ns.targetLevel - Ns.playerLevel) * 0.2 -- 0.2 / 0.4 / 0.6 
	local getCritDefense = (Ns.totalDefense) * 0.04
	local uncritCap = 5
	local critTakenCap = 5.6
	local debuffColor = ""
	local returnText, decimals, liveImmunity
	local resilCrit = GetDodgeBlockParryChanceFromDefense()
	
	chanceGetCrit = (uncritCap + getCritLevel) - resilCrit - Ns.survFittest - Ns.Resilvalue - Ns.SleightTalent - Ns.moltenArmor - Ns.metaCrit
	
	if chanceGetCrit < 0 then chanceGetCrit = 0; debuffColor = Ns.greenText
	elseif chanceGetCrit <= 1.5 then debuffColor = Ns.orangeText
	else debuffColor = Ns.redText end
	
	critTakenCap = critTakenCap - chanceGetCrit
	
	if options.Dec_Def_Crit == 0 then decimals = "%.0f%%"
	elseif options.Dec_Def_Crit == 1 then decimals = "%.1f%%"
	elseif options.Dec_Def_Crit == 2 then decimals = "%.2f%%"
	else decimals = "%.3f%%" end	

	if options.Display_Simulate then liveImmunity = " > " .. (decimals):format(Ns.critReceived)
	else liveImmunity = "" end		
	
	if Ns.Band(EB, Chance) then returnText = debuffColor .. (decimals):format(chanceGetCrit) .. "|r" .. liveImmunity end
	if Ns.Band(EB, Reduction) then returnText = (decimals):format(critTakenCap) .. liveImmunity end
	if Ns.Band(EB, Both) then returnText = debuffColor .. (decimals):format(chanceGetCrit) .. " |r(" .. (decimals):format(critTakenCap) .. ")" .. liveImmunity end		

	HUD:UpdateText(data, returnText)
end
------------------------------------------------

-- Dodge
function Ns.FunctionList.Dodge(HUD, data, options, ...)

	local EB, percentStat, ratingStat = options.Percent_Rating
	local dodgeRating =	GetCombatRating(CR_DODGE)
	local returnText, decimals, liveDodge
	
	if options.Dec_Def_Dodge == 0 then decimals = "%.0f%%"
	elseif options.Dec_Def_Dodge == 1 then decimals = "%.1f%%"
	elseif options.Dec_Def_Dodge == 2 then decimals = "%.2f%%"
	else decimals = "%.3f%%" end

	if options.Display_Simulate then liveDodge = " > " .. (decimals):format(Ns.totalDodge)
	else liveDodge = "" end		
	
	if Ns.Band(EB, Percentage) then returnText = (decimals):format(Ns.dodgeChance) .. liveDodge end
	if Ns.Band(EB, Rating) then returnText = dodgeRating .. liveDodge end
	if Ns.Band(EB, Both) then returnText = (decimals):format(Ns.dodgeChance) .. " (" .. dodgeRating .. ")" .. liveDodge end
	
	HUD:UpdateText(data, returnText)
end
------------------------------------------------

-- Parry
function Ns.FunctionList.Parry(HUD, data, options, ...)

	local EB, percentStat, ratingStat = options.Percent_Rating
	local parryRating =	GetCombatRating(CR_PARRY)
	local returnText, decimals, liveParry

	if options.Dec_Def_Parry == 0 then decimals = "%.0f%%"
	elseif options.Dec_Def_Parry == 1 then decimals = "%.1f%%"
	elseif options.Dec_Def_Parry == 2 then decimals = "%.2f%%"
	else decimals = "%.3f%%" end
	
	if options.Display_Simulate then liveParry = " > " .. (decimals):format(Ns.totalParry)
	else liveParry = "" end			
	
	if Ns.Band(EB, Percentage) then returnText = (decimals):format(Ns.parryChance) .. liveParry end
	if Ns.Band(EB, Rating) then returnText = parryRating .. liveParry end
	if Ns.Band(EB, Both) then returnText = (decimals):format(Ns.parryChance) .. " (" .. parryRating .. ")" .. liveParry end
	
	HUD:UpdateText(data, returnText)
end
------------------------------------------------

-- Block
function Ns.FunctionList.Block(HUD, data, options, ...)

	local EB, percentStat, ratingStat = options.Percent_Rating
	local blockRating =	GetCombatRating(CR_BLOCK)
	local returnText, decimals, liveBlock

	if options.Dec_Def_Block == 0 then decimals = "%.0f%%"
	elseif options.Dec_Def_Block == 1 then decimals = "%.1f%%"
	elseif options.Dec_Def_Block == 2 then decimals = "%.2f%%"
	else decimals = "%.3f%%" end
	
	if options.Display_Simulate then liveBlock = " > " .. (decimals):format(Ns.totalBlock)
	else liveBlock = "" end		
	
	if Ns.Band(EB, Percentage) then returnText = (decimals):format(Ns.blockChance) .. liveBlock end
	if Ns.Band(EB, Rating) then returnText = blockRating .. liveBlock end
	if Ns.Band(EB, Both) then returnText = (decimals):format(Ns.blockChance) .. " (" .. blockRating .. ")" .. liveBlock end
	
	HUD:UpdateText(data, returnText)
end
------------------------------------------------

-- Avoidance
function Ns.FunctionList.Avoidance(HUD, data, options, ...)
	
	local EB = options.Enhanced_Base
	local returnText, decimals
	local avoidColor = ""
	
	if Ns.avoid >= 102.4 then avoidColor = Ns.greenText
	else avoidColor = "" end
	
	if options.Dec_Def_Avoidance == 0 then decimals = "%.0f%%"
	elseif options.Dec_Def_Avoidance == 1 then decimals = "%.1f%%"
	elseif options.Dec_Def_Avoidance == 2 then decimals = "%.2f%%"
	else decimals = "%.3f%%" end
	
	if Ns.Band(EB, Enhanced) then returnText = avoidColor .. (decimals):format(Ns.avoid) end
	if Ns.Band(EB, Basic) then returnText = avoidColor .. (decimals):format(Ns.avoidBase) end	
	if Ns.Band(EB, Both) then returnText = avoidColor .. (decimals):format(Ns.avoid) .. " |r(" .. avoidColor .. (decimals):format(Ns.avoidBase) .. ")" end	

	HUD:UpdateText(data, returnText)
end
------------------------------------------------

-- Crushing
function Ns.FunctionList.Crushing(HUD, data, options, ...)
	
	local EB = options.Enhanced_Base
	local returnText, decimals
	local crushColor = ""
	local targetCrushLevel = UnitLevel("target")
	
	if targetCrushLevel == 0 then targetCrushLevel = Ns.playerLevel + 4 end

	local crush = ((targetCrushLevel * 5) - Ns.totalDefense) * 2 - 15	

	if crush <= 0 then crushColor = Ns.greenText; crush = 0
	else crushColor = "" end
	
	if options.Dec_Def_Crush == 0 then decimals = "%.0f%%"
	elseif options.Dec_Def_Crush == 1 then decimals = "%.1f%%"
	elseif options.Dec_Def_Crush == 2 then decimals = "%.2f%%"
	else decimals = "%.3f%%" end	
	
	if Ns.Band(EB, Enhanced) then returnText = crushColor .. (decimals):format(crush) end
	if Ns.Band(EB, Basic) then returnText = crushColor .. (decimals):format(crush) end	
	if Ns.Band(EB, Both) then returnText = crushColor .. (decimals):format(crush) .. " |r(" .. crushColor .. (decimals):format(crush) .. ")" end		

	HUD:UpdateText(data, returnText)
end
------------------------------------------------
